*********************************************************************
* Mandelbrot.asm    Mandelbrot & Julia fractal set generator
* Author:           Ken Murray
* Notes:            Most comments removed due to QL space limitations
*********************************************************************
LF        equ       10
CR        equ       13
SPACE     equ       32
ESC       equ       27
TRUE      equ       1
FALSE     equ       0

          bra       start               ;Start of Code space
          dc.w      $4AFB               ;ID word
          dc.w      9
          dc.b      'MANDELBROT'
          dc.b      'K Murray'

start     lea       accurate(pc),A4
          move.w    #255,(A4)
          lea       multvar(pc),A4
          move.w    #1,(A4)
          moveq     #$10,D0
          moveq     #8,D1
          moveq     #-1,D2
          trap      #1
          lea       main_win(pc),A1
          move.w    $C8,A0
          jsr       (A0)
          tst.b     D0
          bne       kill
          lea       mainchan(pc),A1
          move.l    A0,(A1)

re_start  lea       rp(pc),A3           ;rp = -2
          move.l    #$20000000,D1
          neg.l     D1
          move.l    D1,(A3)+
          move.l    #$13000000,D1       ;ip = -1.1875
          neg.l     D1
          move.l    D1,(A3)+
          move.l    #$26000000,D2       ;size of 2.375 (ie 2*ip)
          move.l    D2,(A3)+
          move.l    #0,(A3)+            ;creal
          move.l    #0,(A3)+            ;cimag
          bsr       memoryscr

*************************
main_seq  lea       inhibit(pc),A4
          move.w    #0,(A4)
          bsr       mult0task
          bsr       mainmenu
          cmpi.w    #9,D7
          beq       killjob
          bsr       multtask1
          cmpi.w    #3,D7
          beq       pointer
          cmpi.w    #4,D7
          beq       magnify
          cmpi.w    #5,D7
          beq       mandbrot
          cmpi.w    #7,D7
          beq       pausequit
          cmpi.w    #8,D7
          beq       re_start

pausequit moveq     #3,D1
          bsr       delay
          bsr       keyrow0
          cmpi.b    #32,D1              ;F5 key?
          beq       main_seq
          bsr       keyrow1
          cmpi.b    #1,D1
          beq       main_seq
          bra       pausequit

killjob   lea       mainchan(pc),A1     ;and RETURN TO SUPERBASIC
          move.l    (A1),A0
          bsr       closer
kill      moveq     #-1,D1
          clr.l     D3
          moveq     #5,D0
          trap      #1

************************

sub_sequ  bsr       mult0task
          bsr       mainmenu
          cmpi.w    #9,D7
          beq       killseq
          bsr       multtask1
          cmpi.w    #8,D7
          beq       re_start
subseqx   bra       loopcont
killseq   add.l     #4,A7
          bra       killjob

memoryscr lea       mem_scr(pc),A2      ;add to screen driver list
          bsr       ext_driv            ;and perform
          lea       scr_mand(pc),A2
          move.b    #TRUE,(A2)
          rts
mem_scr   lea       memory(pc),A3
          move.l    #$20000,A2
          move.l    #$7FFF,D1
videoloop move.b    (A3)+,(A2)+         ;Memory to Screen
          dbf       D1,videoloop
          clr.l     D0                  ;No error
          rts

clearscr  lea       clr_scr(pc),A2
          bsr       ext_driv
          rts
clr_scr   move.l    #$7FFF,D1           ;no. of bytes to sclear
          move.l    #$20000,A2
clearloop move.b    #$00,(A2)+          ;clear video memory
          dbf       D1,clearloop
          clr.l     D0
          rts

pointer   moveq     #40,D5
          moveq     #30,D6
usekey0   bsr       altcross
          movem.w   D5/D6,-(A7)
          moveq     #0,D1
          bsr       delay
          movem.w   (A7)+,D5/D6
usekey1   bsr       keyrow2
          cmpi.b    #64,D1
          beq       useexit             ;M
          bsr       keyrow1
          cmpi.b    #0,D1
          beq       usekey1
          cmpi.b    #2,D1
          bne       useup
          cmpi.w    #4,D5
          beq       usekey1
          bsr       altcross
          sub.w     #1,D5
          bra       usekey0
useup     cmpi.b    #4,D1
          bne       useright
          cmpi.w    #251,D6
          beq       usekey1
          bsr       altcross
          add.w     #1,D6
          bra       usekey0
useright  cmpi.b    #16,D1
          bne       usedown
          cmpi.w    #251,D5
          beq       usekey1
          bsr       altcross
          add.w     #1,D5
          bra       usekey0
usedown   cmpi.w    #128,D1
          bne       usekey1
          cmpi.w    #4,D6
          beq       usekey1
          bsr       altcross
          sub.w     #1,D6
          bra       usekey0
useexit   bsr       altcross            ;delete cross
          lea       xorig(pc),A2
          move.w    D5,(A2)
          lea       yorig(pc),A2
          move.w    D6,(A2)
          bsr       findxy
          lea       size(pc),A2
          move.l    #$2E900000,(A2)
          lea       rp(pc),A2
          lea       creal(pc),A3
          move.l    (A2)+,(A3)+
          move.l    (A2),(A3)
          move.l    #$17480000,D1
          neg.l     D1
          move.l    D1,(A2)
          move.l    #$20000000,D1
          neg.l     D1
          move.l    D1,-(A2)
          bra       mandbrot

altcross  movem.w   D5/D6,-(A7)
          move.w    D5,D1               ;X side
          move.w    D6,D2               ;Y side
          lea       cross(pc),A2
          bsr       ext_driv
          movem.w   (A7)+,D5/D6
          rts
cross     moveq     #$7,D3
          moveq     #4,D4
crossxy   add.w     D4,D1
          bsr       plotor
          sub.w     D4,D1
          sub.w     D4,D1
          bsr       plotor
          add.w     D4,D1
          add.w     D4,D2
          bsr       plotor
          sub.w     D4,D2
          sub.w     D4,D2
          bsr       plotor
          add.w     D4,D2
          subq.w    #1,D4
          bne       crossxy
          clr.l     D0
          rts

magnify   lea       side(pc),A1         ;set start  parameters
          moveq     #50,D4
          move.l    D4,D3
          move.w    D4,(A1)             ;for rectangle
          lea       xorig(pc),A1
          moveq     #40,D5
          move.w    D5,(A1)
          lea       yorig(pc),A1
          moveq     #30,D6
          move.w    D6,(A1)
          bsr       magstart
          lea       side(pc),A1
          move.w    D4,(A1)
          lea       xorig(pc),A1
          move.w    D5,(A1)
          lea       yorig(pc),A1
          move.w    D6,(A1)
          bra       starter   ;NOW DRAW MANDELBROT

magstart  bsr       altersqu
magner    bsr       usekey
          cmpi.b    #64,D1
          beq       magnerx
          cmpi.b    #0,D1
          bne       magmove
          bsr       sideredu
          cmpi.b    #0,D1
          beq       magner
magmove   move.l    D4,D3
          bsr       altersqu
          bra       magner
magnerx   bsr       altersqu
          rts

usekey    bsr       keyrow2
          cmpi.b    #64,D1
          beq       magexit0
          bsr       keyrow1
          cmpi.b    #0,D1
          beq       usekey
          cmpi.b    #2,D1
          bne       magup
          cmpi.w    #0,D5
          beq       magexit
          bsr       altersqu
          sub.w     #1,D5
magexit0  rts
magup     cmpi.b    #4,D1
          bne       magright
          move.w    D6,D2
          add.w     D4,D2
          cmpi.w    #255,D2
          beq       magexit
          bsr       altersqu
          add.w     #1,D6
          rts
magright  cmpi.b    #16,D1
          bne       magdown
          move.w    D5,D2
          add.w     D3,D2
          cmpi.w    #255,D2
          beq       magexit
          bsr       altersqu
          add.w     #1,D5
          rts
magdown   cmpi.b    #128,D1
          bne       magexit
          cmpi.w    #0,D6
          beq       magexit
          bsr       altersqu
          sub.w     #1,D6
          rts
magexit   clr.l     D1
          rts

sideredu  bsr       keyrow1
          cmpi.b    #64,D1
          bne       sideinc
          cmpi.w    #5,D4
          beq       sideexit
          bsr       altersqu
          sub.w     #1,D3
          sub.w     #1,D4
          rts
sideinc   cmpi.b    #32,D1
          bne       sideexit
          cmpi.w    #200,D4
          beq       sideexit
          move.w    D6,D2
          add.w     D4,D2
          cmpi.w    #255,D2
          beq       sideexit
          move.w    D5,D2
          add.w     D3,D2
          cmpi.w    #255,D2
          beq       sideexit
          bsr       altersqu
          add.w     #1,D3
          add.w     #1,D4
          rts
sideexit  clr.l     D1
          rts

altersqu  movem.l   D1-D6,-(A7)
          move.w    D3,D1
          move.w    D4,D2
          swap      D1
          swap      D2
          move.w    D5,D1
          move.w    D6,D2
          lea       box(pc),A2
          bsr       ext_driv
          movem.l   (A7)+,D1-D6
          rts
box       clr.l     D4
          clr.l     D5
          move.l    D1,D4
          move.l    D2,D5
          swap      D4
          subq.w    #1,D4
          swap      D5
          subq.w    #1,D5
          clr.l     D7
          move.l    #$7,D3
          move.w    D4,D7
squarex   bsr       plotor
          add.w     D5,D2
          bsr       plotor
          sub.w     D5,D2
          addq.w    #1,D1
          dbf       D7,squarex
          subq.w    #1,D1
          clr.l     D7
          move.w    D5,D7
squarey   bsr       plotor
          sub.w     D4,D1
          bsr       plotor
          add.w     D4,D1
          addq.w    #1,D2
          dbf       D7,squarey
          clr.l     D0                  ;no error
          rts

xorig     ds.w      1
yorig     ds.w      1
side      ds.w      1

rp        ds.l      1
ip        ds.l      1
size      ds.l      1
creal     ds.l      1
cimag     ds.l      1
tempz     ds.l      1
accurate  ds.w      1

*** CONVERT MAGNIFY RECTANGLES CO-ORDS TO MANDELBROTS PARAMETERS ***
starter   bsr       findxy
          bsr       reset               ;A1 = BOS
          sub.l     #2,A1
          lea       side(pc),A3
          move.w    0(A3),0(A6,A1.L)      ;make it fp.
          bsr       floater
          sub.l     #2,A1
          move.w    #256,0(A6,A1.L)
          bsr       floater
          bsr       divide              ;stack = f.p. ratio (side/256)
          sub.l     #6,A1
          lea       size(pc),A3
          move.l    (A3),D1
          bsr       intfloat
          bsr       multip
          bsr       flo2int
          lea       size(pc),A3
          move.l    D7,(A3)             ;new size = old size * ratio
          bra       mandbrot

findxy    bsr       findxgap            ;Find old X_gap
          lea       xorig(pc),A3
          sub.l     #2,A1
          move.w    0(A3),0(A6,A1.L)
          bsr       floater
          bsr       multip              ;offset = old x_gap * xorig
          bsr       flo2int
          lea       rp(pc),A3
          add.l     D7,(A3)             ;new rp = old rp + offset

          bsr       findygap            ;Find old Y_gap
          lea       yorig(pc),A3
          sub.l     #2,A1
          move.w    0(A3),0(A6,A1.L)
          bsr       floater
          bsr       multip              ;offset = old y_gap * yorig
          bsr       flo2int
          lea       ip(pc),A3
          add.l     D7,(A3)             ;new ip = old ip + offset
          rts

findxgap  bsr       findygap            ;Put Y_gap on stack & multiply
          move.l    #$16000000,D1       ;by scale factor of 1.375
          sub.l     #6,A1
          bsr       intfloat
          bsr       multip              ;X_gap on stack
          rts
findygap  bsr       reset
          lea       size(pc),A3
          move.l    (A3),D1
          sub.l     #6,A1               ;make room for size
          bsr       intfloat
          sub.l     #2,A1
          move.w    #256,0(A6,A1.L)
          bsr       floater
          bsr       divide              ;leaving Y_gap on stack
          rts

mandbrot  bsr       findxgap            ;Magnify by Value enters here
          lea       x_gap(pc),A3
          move.w    0(A6,A1.L),0(A3)
          move.l    2(A6,A1.L),2(A3)
          bsr       findygap
          lea       y_gap(pc),A3
          move.w    0(A6,A1.L),0(A3)
          move.l    2(A6,A1.L),2(A3)

*****  MANDELBROT CALCULATION PROCEDURE  *****
mandel    lea       inhibit(pc),A5
          move.w    #1,(A5)
          bsr       clearscr
          bsr       reset
          lea       scr_mand(pc),A5
          lea       sel_mand(pc),A3
          move.b    (A3),(A5)
          lea       mI(pc),A3
          move.w    #-1,(A3)            ;zero mI counter

loopm     lea       mI(pc),A3
          addq.w    #1,(A3)             ;start of loop MI
          bsr       reset               ;set A1 to BOS
          subq.l    #2,A1
          move.w    0(A3),0(A6,A1.L)      ;mI
          bsr       floater
          subq.l    #6,A1
          lea       y_gap(pc),A2
          move.w    0(A2),0(A6,A1.L)
          move.l    2(A2),2(A6,A1.L)
          bsr       multip
          bsr       flo2int
          lea       ip(pc),A4
          add.l     (A4),D7
          lea       ic(pc),A5
          move.l    D7,(A5)             ;store ic
*  ic = mI * gap + ip

          lea       nI(pc),A5
          move.w    #-1,(A5)            ;zero nI counter

loopn     lea       nI(pc),A5
          addq.w    #1,(A5)             ;start of loop nI
          lea       mI(pc),A4
          cmpi.w    #1,(A4)
          bge       skipsec
          bsr       reset               ;set up rc coord array
          subq.l    #2,A1
          move.w    0(A5),0(A6,A1.L)      ;nI
          bsr       floater
          subq.l    #6,A1
          lea       x_gap(pc),A2
          move.w    0(A2),0(A6,A1.L)
          move.l    2(A2),2(A6,A1.L)
          bsr       multip
          bsr       flo2int
          lea       rp(pc),A4
          add.l     (A4),D7
          lea       rc(pc),A5           ;put rc into array rc
          lea       nI(pc),A3
          clr.l     D1
          move.w    (A3),D1
          lsl.l     #2,D1               ;count * 4 bytes (ie Long)
          move.l    D7,0(A5,D1.L)        ;store rc
*  rc = nI * gap + rp

skipsec   bsr       keyrow0             ;test F5 key
          cmpi.b    #32,D1              ;F5
          bne       loopcont
          bra       sub_sequ            ;call to MENU

loopcont  lea       rc(pc),A3
          lea       nI(pc),A4
          clr.l     D0
          move.w    (A4),D0
          lsl.l     #2,D0
          move.l    0(A3,D0.L),A1
          lea       scr_mand(pc),A3
          move.b    (A3),D0
          lea       ic(pc),A3
          move.l    (A3),A2
          lea       accurate(pc),A3
          move.w    (A3),A5
          sub.l     A0,A0               ;counter
          sub.l     A4,A4               ;reset rzsqu
          sub.l     A3,A3               ;reset izsqu
          moveq     #0,D5               ;rz
          moveq     #0,D4               ;iz
          moveq     #11,D6              ;used for shifts
          cmpi.b    #TRUE,D0
          beq       repeater
          exg       D5,A1
          lea       creal(pc),A1
          move.l    (A1),A1
          exg       D4,A2
          lea       cimag(pc),A2
          move.l    (A2),A2
          bra       julia

*** repeater is performed a maximum of <accurate> times ***
repeater  addq.l    #1,A0               ;increment counter
          clr.l     D7                  ;sign flag
          move.l    D5,D0               ;rz
          bpl       rep1
          moveq     #1,D7               ;negative
          neg.l     D0
rep1      move.l    D4,D1               ;iz
          bpl       rep2
          addq.l    #1,D7               ;negative
          neg.l     D1
rep2      move.w    D0,D4               ;2*rz*iz calculation
          mulu      D1,D4               ;B*D
          swap      D4
          andi.l    #$FFFF,D4           ;Total = BD=>16
          swap      D0
          move.w    D1,D3
          mulu      D0,D3               ;A*D
          move.l    D0,D2
          swap      D2
          swap      D1
          mulu      D1,D2               ;B*C
          add.l     D2,D4
          add.l     D3,D4
          lsr.l     D6,D4
          mulu      D1,D0               ;A*C
          lsl.l     #5,D0
          add.l     D0,D4     ;total=(5<=AC)+((BC+AD+(DB=>16))=>6)
          cmpi.b    #1,D7               ;if exactly one negative number
          bne       rep3                ;then answer is negative
          neg.l     D4
rep3      add.l     A2,D4               ;iz=iz+ic
          suba.l    A3,A4
          adda.l    A1,A4
          move.l    A4,D5               ;rz = rzsqu - izsqu + rc

julia     move.l    D5,D0               ;square rz
          bpl       rep4
          neg.l     D0
rep4      bsr       squaring            ;squares number in D0
          movea.l   D0,A4               ;=> rzsqu

          move.l    D4,D0               ;square iz
          bpl       rep5
          neg.l     D0
rep5      bsr       squaring  ;If Overflow, it will exit to modulus
          movea.l   D0,A3               ;=> izsqu

          cmpa.l    A5,A0
          bhi       modulus             ;iterations > accuracy ?
          add.l     A4,D0
          cmpi.l    #$40000000,D0       ;rzsqu + izsqu > 4 ?
          bmi       repeater

modulus   move.l    A0,D3
          andi.l    #7,D3               ;make count = 0 to 7
          beq       plot_not
          lea       nI(pc),A3
          move.w    (A3),D1             ;X co-ord
          lea       mI(pc),A5
          move.w    (A5),D2             ;Y co-ord
          bsr       plotand             ;overwrite
plot_not  lea       nI(pc),A3
          cmpi.w    #254,(A3) ;END FOR nI
          blt       loopn

          lea       mI(pc),A5
          cmpi.w    #254,(A5) ;END FOR mI
          blt       loopm

          clr.l     D0
          bra       main_seq  ;END OF MANDELBROT CALCULATION


*Arithmetic routines using 32 bit signed, two's complemented routines

squaring  move.w    D0,D1               ;(a+b)^2=a^2+2ab+b^2
          mulu      D0,D1               ;B*B
          swap      D1
          andi.l    #$FFFF,D1
          lsr.w     #1,D1               ;BB=>17
          move.w    D0,D2
          swap      D0
          mulu      D0,D2               ;A*B
          add.l     D2,D1
          lsr.l     D6,D1               ;(AB+(BB=>17))=>11
          mulu      D0,D0               ;A*A
          cmpi.l    #$04000000,D0
          bpl       exit_squ            ;overflow?
          lsl.l     #4,D0
          add.l     D1,D0
          rts
exit_squ  add.l     #4,A7               ;remove RTS address
          bra       modulus             ;and EXIT

*Arithmetic routines for floating point
multip    move.w    #$0E,D0
          bra       maths
divide    move.w    #$10,D0
          bra       maths
adder     moveq     #$0A,D0
          bra       maths
subtract  moveq     #$0C,D0
          bra       maths
integer   moveq     #$04,D0
          bra       maths
intlong   moveq     #$06,D0
          bra       maths
floater   moveq     #$08,D0
          bra       maths
rooter    moveq     #$28,D0
          bra       maths
duplic    moveq     #$16,D0
          bra       maths
* D0=maths op required
maths     movem.l   D1-D7/A2-A5,-(A7)
          moveq     #0,D7
          movea.w   $11C,A0
          jsr       (A0)
          movem.l   (A7)+,D1-D7/A2-A5
          rts

reset     lea       stack(pc),A1
          sub.l     A6,A1
          rts

* A5 = ascII string to be coverted to integer in D7.l
asc2int   clr.l     D7
          move.l    A5,A0
          move.w    (A0)+,D7
          sub.l     A6,A0
          add.l     A0,D7
          move.w    $100,A3
          jsr       (A3)
          cmpi.w    #0,D0
          bne       asc2intx
flo2int   subq.l    #2,A1
          move.w    #16384,0(A6,A1.L)
          bsr       floater
          bsr       multip
          subq.l    #2,A1
          move.w    #16384,0(A6,A1.L)
          bsr       floater
          bsr       multip
          bsr       intlong
          cmpi.w    #0,D0
          bne       asc2intfx
          move.l    0(A6,A1.L),D7
          addq.l    #4,A1
          bra       asc2intx
asc2intfx addq.l    #6,A1
asc2intx  rts

* D7 = integer to be converted to ascII string at A5
int2asc   move.l    A5,-(A7)
          subq.l    #6,A1
          move.l    D7,D1
          btst.l    #31,D1
          bne       int_neg
          addq.l    #1,A5
int_neg   bsr       reset
          bsr       intfloat
          bsr       reset
          clr.l     D1
          move.l    A5,A0
          sub.l     A6,A0
          move.w    $F0,A3
          jsr       (A3)
          move.l    (A7),A5
          add.l     #1,A5
find_E    cmpi.b    #"E",0(A5,D1.L)
          beq       normalise
          dbf       D1,find_E
normexit  move.l    (A7)+,A5
          rts
* Converts scientific notation to normal ie. '6.9321E-2' to '.069321'
normalise move.b    2(A5,D1.L),D4
          move.b    #'0',2(A5,D1.L)
          move.b    #'0',1(A5,D1.L)
          move.b    #'0',0(A5,D1.L)
          subq.b    #1,D1
          sub.b     #48,D4
          beq       normexit
          cmpi.b    #'.',(A5)
          beq       normok
          cmpi.b    #0,D1
          bne       haspoint
          move.b    #'.',1(A5)
          addq.b    #1,D1
haspoint  move.b    (A5),1(A5)
          move.b    #'.',(A5)
          subq.b    #1,D4
          beq       normexit
normok    subq.b    #1,D4
          bmi       normexit
          add.l     #1,A5
          moveq     #7,D5
loopright move.b    0(A5,D5.L),1(A5,D5.L)
          dbf       D5,loopright
          move.b    #'0',(A5)
          bra       normok

* convert signed real integer (D1) to floating point equivalent (D1.w,D2.l)
intfloat  moveq     #8,D2
          btst.l    #31,D1
          beq       plus_int
          neg.l     D1
          neg.w     D2
plus_int  move.w    #0,0(A6,A1.L)
          move.l    #0,2(A6,A1.L)
          lsl.l     #1,D1
          sub.l     #2,A1
          move.w    D2,0(A6,A1.L)
          bsr       floater
          moveq     #30,D2
addornot  sub.l     #2,A1
          move.w    #2,0(A6,A1.L)
          bsr       floater
          bsr       divide
          lsl.l     #1,D1
          bcc       notadd
          bsr       duplic
          add.l     #6,A1
          bsr       adder
          sub.l     #12,A1
          move.w    0(A6,A1.L),6(A6,A1.L)
          move.l    2(A6,A1.L),8(A6,A1.L)
          add.l     #6,A1
notadd    dbf       D2,addornot
          add.l     #6,A1
          move.w    0(A6,A1.L),D1
          move.l    2(A6,A1.L),D2
          rts

*** plot point x,y on screen in colour ***
plotand   lea       andplot(pc),A2
          move.l    D3,A1
ext_driv  moveq     #-1,D3
          lea       multvar(pc),A0
          cmpi.w    #1,(A0)
          beq       tryplot
          moveq     #0,D3
tryplot   lea       mainchan(pc),A0
          move.l    (A0),A0
          moveq     #9,D0
          trap      #3
          tst.b     D0
          bne       tryplot
          rts

andplot   move.l    A1,D3
          bsr       point
          and.w     D2,(A1)
          or.w      D3,(A1)
          clr.l     D0
          rts
plotor    movem.l   D1-D7,-(A7)
          bsr       point
          eor.w     D3,(A1)
          movem.l   (A7)+,D1-D7
          clr.l     D0
          rts

point     andi.l    #7,D3
          eori.w    #255,D2
          lsl.l     #1,D1
          move.l    D1,D0
          lea       colour(pc),A1
          lsl.w     #1,D3
          move.w    0(A1,D3.W),D3
          movea.l   #$20000,A1
          lsr.w     #2,D1
          andi.l    #$7E,D1
          adda.w    D1,A1
          lsl.w     #7,D2
          adda.w    D2,A1
          andi.l    #6,D0
          lsr.w     D0,D3
          move.l    #%0111111100111111,D2
          ror.w     D0,D2
          rts

colour    dc.w      0
          dc.w      %0000000001000000
          dc.w      %0000000010000000
          dc.w      %0000000011000000
          dc.w      %1000000000000000
          dc.w      %1000000001000000
          dc.w      %1000000010000000
          dc.w      %1000000011000000

pixcalc   ext.l     D2                  ;calculate address of point x,y
          lsr.w     #2,D1               ;(used by move menu)
          andi.l    #$007E,D1
          move.l    #$20000,A1
          adda.w    D1,A1
          lsl.w     #7,D2
          adda.w    D2,A1
          rts

* variables for int program
rc        ds.l      256
rz        ds.l      1
ic        ds.l      1
iz        ds.l      1
mI        ds.w      1                   ;Y counter
nI        ds.w      1                   ;X counter
scr_mand  ds.b      1
sel_mand  ds.b      1
* variables for f.p. section
x_gap     ds.w      3
y_gap     ds.w      3
          ds.l      20
stack     ds.w      3                   ;BOS

************************
mainmenu  bsr       swap
          lea       sel_mand(pc),A4     ;set selection to current screen
          lea       scr_mand(pc),A5
          move.b    (A5),(A4)
          bsr       openchan
selectrep bsr       printF5
          lea       curtoplim(pc),A2    ;top limit of bar
          move.w    #1,(A2)
          lea       curbotlim(pc),A2    ;bottom limit
          move.w    #9,(A2)
          lea       cursorpos(pc),A2
          move.w    #1,(A2)
          bsr       select              ;return selection in cursorpos=A2
          move.w    (A2),D7
          cmpi.w    #7,D7
          beq       exitmain
          cmpi.w    #8,D7
          beq       exitmain
          cmpi.w    #9,D7
          beq       exitmain
          bsr       othermenu
          cmpi.w    #3,D7
          beq       exitmain
          cmpi.w    #4,D7
          beq       exitmain
          cmpi.w    #5,D7
          beq       exitmain
          bra       selectrep
exitmain  bsr       closer
          bsr       swap
          rts
************************

** Swap screen section to buffer and vice versa **
swap      movem.l   D0/D1/A0/A5,-(A7)
          lea       wincoord(pc),A1
          move.w    (A1)+,D1
          move.w    (A1),D2
          bsr       pixcalc
          lea       menuswap(pc),A2
          moveq     #-1,D3
          bsr       tryplot
          movem.l   (A7)+,D0/D1/A0/A5
          rts
menuswap  lea       menubuff(pc),A5
          move.l    #101,D0
repeat    move.l    #21,D1
repeat1   move.w    (A5),D2
          move.w    (A1),(A5)+
          move.w    D2,(A1)+
          dbf       D1,repeat1
          adda.l    #84,A1
          dbf       D0,repeat
          clr.l     D0
          rts

mult0task lea       multvar(pc),A5
          cmpi.w    #1,(A5)
          beq       mult0exit
          move.l    (A7)+,A5
          andi.w    #$D8FF,SR
          move.l    A5,-(A7)
mult0exit rts
multtask1 lea       multvar(pc),A5
          cmpi.w    #1,(A5)
          beq       mult1exit
          move.l    (A7)+,A5
          trap      #0
          ori.w     #$0700,SR
          move.l    A5,-(A7)
mult1exit rts

printF5   lea       messF5(pc),A5
          add.l     #52,A5              ;offset
          moveq     #3,D1
          lea       mandsel(pc),A4
          lea       sel_mand(pc),A3
          cmpi.b    #TRUE,(A3)
          beq       F5mand
          lea       julisel(pc),A4
F5juli    move.b    (A4)+,(A5)+
          dbf       D1,F5juli
          bra       F5exit
F5mand    move.b    (A4)+,(A5)+
          dbf       D1,F5mand
F5exit    lea       messF5(pc),A5
          bsr       print
          moveq     #10,D1
          moveq     #1,D2
accoffset lea       accurate(pc),A5
          move.l    #$10,D0
          cmpi.w    #99,(A5)
          bgt       cursoradd
          addq.w    #1,D1
cursoradd cmpi.w    #9,(A5)
          bgt       curadd1
          addq.w    #1,D1
curadd1   moveq     #-1,D3
          trap      #3
          move.w    (A5),D1
          move.w    $CE,A3
          jsr       (A3)
          rts

openchan  lea       chanwin(pc),A1
          move.w    $C8,A0
          jsr       (A0)
          rts
*** entry: A5 = length.w & start of message to be printed on screen ***
print     move.l    A0,-(A7)
          move.l    #$2C,D0
          moveq     #1,D1
          moveq     #-1,D3
          trap      #3
          move.l    #$20,D0             ;SD.CLEAR - clear window
          moveq     #-1,D3
          trap      #3
readmess  lea       space14(pc),A1
          moveq     #14,D1
          cmpi.b    #'$',(A5)
          beq       readexit
          cmpi.b    #'@',(A5)
          bne       prt14spc
          add.l     #1,A5
          bsr       prtstr
          bra       readmess
prt14spc  exg       A5,A1
          bsr       prtstr
          exg       A5,A1
          bra       readmess
readexit  move.l    (A7)+,A0
          rts
closer    moveq     #2,D0               ;IO.CLOSE - close channel
          trap      #2
          rts                           ;BACK TO PROG

othermenu cmpi.w    #1,D7
          beq       accumenu
          cmpi.w    #2,D7
          beq       filemenu
          cmpi.w    #5,D7
          beq       movemenu
          cmpi.w    #6,D7
          beq       multmenu
          lea       inhibit(pc),A5
          cmpi.w    #0,(A5)
          bne       otherexit
          cmpi.w    #3,D7
          beq       funcmenu
          cmpi.w    #4,D7
          beq       magnmenu
otherexit clr.l     D7                  ;Else return to main menu
          rts

accumenu  lea       menuaccu(pc),A5
          bsr       print
rew       moveq     #0,D1
          bsr       delay
          moveq     #5,D1
          moveq     #6,D2
          bsr       positcur
          moveq     #$23,D0
          moveq     #-1,D3
          trap      #3
          move.w    #5,D1               ;x
          move.w    #6,D2               ;y
          bsr       accoffset           ;print accuracy with offset
rewagain  moveq     #1,D1
          bsr       delay
          bsr       keyrow1
          cmpi.b    #0,D1
          beq       rewagain
          lea       accurate(pc),A5
          cmpi.b    #1,D1
          beq       exitaccu            ;enter
          cmpi.b    #4,D1
          bne       downacc
          cmpi.w    #255,(A5)
          beq       rew                 ;avoid >255
          addi.w    #8,(A5)
          bra       rew
downacc   cmpi.b    #128,D1
          bne       rewagain
          cmpi.w    #7,(A5)
          beq       rew                 ;avoid <7
          subi.w    #8,(A5)
          bra       rew
exitaccu  clr.l     D7
          rts

movemenu  lea       menumove(pc),A5
          bsr       print
keymove   bsr       keyrow1
          cmpi.b    #1,D1
          bne       keymove             ;Enter
          bsr       closer
          bsr       swap
          clr.l     D3
          clr.l     D4
          clr.l     D5
          clr.l     D6
          lea       wincoord(pc),A1     ;open menus' co-ords
          move.w    (A1)+,D5
          lsr.l     #1,D5
          move.w    (A1),D6
          add.w     #102,D6
          eori.b    #255,D6
          move.w    #88,D3
          move.w    #102,D4
          bsr       altersqu
movener   bsr       usekey
          cmpi.b    #64,D1
          beq       moveexit
          cmpi.b    #0,D1
          beq       movener
          bsr       altersqu
          bra       movener
moveexit  bsr       altersqu
          add.w     #102,D6
          eori.b    #255,D6
          lea       wincoord(pc),A1
          lsr.l     #2,D5
          lsl.l     #3,D5
          movem.w   D5/D6,-(A7)
          move.w    D5,(A1)+
          move.w    D6,(A1)
          movem.w   (A7)+,D5/D6
          lea       concoord(pc),A1
          add.w     #2,D5
          add.w     #77,D6
          move.w    D5,(A1)+
          move.w    D6,(A1)
          bsr       swap
          bsr       openchan
          clr.l     D7
          rts

funcmenu  lea       sel_mand(pc),A4     ;toggle Mand/Julia
          lea       scr_mand(pc),A3
          move.b    (A4),D0
          add.b     (A3),D0
          tst.b     D0
          beq       funcexit            ;exit if scr=false & sel=false
          eor.b     #1,(A4)
funcexit  clr.l     D7
          rts

magnmenu  lea       menumagn(pc),A5     ;Menu to select magnify option
          lea       sel_mand(pc),A2
          cmpi.b    #TRUE,(A2)
          beq       mand_pic
          lea       scr_mand(pc),A2
          cmpi.b    #FALSE,(A2)
          beq       mand_pic
          lea       menu4mag(pc),A5     ;else Julia mag screen
mand_pic  bsr       print
          lea       curtoplim(pc),A3
          move.w    #2,(A3)
          lea       curbotlim(pc),A3
          move.w    #4,(A3)
          lea       cursorpos(pc),A2
          move.w    #2,(A2)
          bsr       select
          move.w    (A2),D7             ;selection
          cmpi.w    #2,D7
          beq       mag1menu            ;Location menu
          cmpi.w    #3,D7
          beq       mag2menu            ;Rectangle/Pointer menu
          bra       mag3menu            ;Values

mag1menu  lea       sel_mand(pc),A4
          lea       menu1mag(pc),A5     ;default
          cmpi.b    #TRUE,(A4)          ;julias Creal & Cimag
          beq       magskip1
          lea       scr_mand(pc),A3
          cmpi.b    #TRUE,(A3)
          beq       magskip0
          lea       menu5mag(pc),A5
          bsr       risnumb
          lea       menu5mag(pc),A5
          bra       magskip2
magskip0  move.b    #TRUE,(A4)          ;sel_mand=true
          lea       menu1mag(pc),A5
          bsr       risnumb
          lea       menu1mag(pc),A5     ;restore
          lea       sel_mand(pc),A4
          move.b    #FALSE,(A4)
          bra       magskip2
magskip1  lea       menu1mag(pc),A5
          bsr       risnumb
          lea       menu1mag(pc),A5
magskip2  bsr       print               ;print menu with updated values
          moveq     #2,D1
          bsr       delay
          bsr       key_enter
          clr.l     D7
          rts
mag2menu  lea       menu2mag(pc),A5
          moveq     #4,D7
          lea       sel_mand(pc),A4
          cmpi.b    #TRUE,(A4)
          beq       mag2skip
          lea       scr_mand(pc),A4
          cmpi.b    #FALSE,(A4)
          beq       mag2skip
          lea       menu6mag(pc),A5
          moveq     #3,D7
mag2skip  move.l    D7,-(A7)
          bsr       print               ;magnify rectangle/pointer menu
          moveq     #2,D1
          bsr       delay
          bsr       key_enter
          move.l    (A7)+,D7
          rts
mag3menu  lea       curtoplim(pc),A3
          move.w    #2,(A3)
mag3rep   lea       curbotlim(pc),A3
          move.w    #5,(A3)
          lea       sel_mand(pc),A4
          cmpi.b    #TRUE,(A4)
          beq       mag3ret
          move.w    #7,(A3)
          lea       menu7mag(pc),A5     ;julia
          bsr       risnumb
          lea       menu7mag(pc),A5
          bra       mag3skp
mag3ret   lea       menu3mag(pc),A5     ;mandelbrot
          bsr       risnumb
          lea       menu3mag(pc),A5
mag3skp   bsr       print               ;print menu
          lea       cursorpos(pc),A2
          move.w    #2,(A2)             ;reset bar to Top pos.
          bsr       select              ;Real/Imag/Size/Plot?
          bsr       reset
          cmpi.w    #2,(A2)             ;Select Real
          bne       mag31
          bsr       getstr
          bne       mag3rep             ;If error, exit back
          lea       rp(pc),A3
          move.l    D7,(A3)
          bra       mag3rep
mag31     cmpi.w    #3,(A2)             ;Select Imag
          bne       mag32
          bsr       getstr
          bne       mag3rep             ;error cond., exit
          lea       ip(pc),A3
          move.l    D7,(A3)
          bra       mag3rep
mag32     cmpi.w    #4,(A2)             ;Select Size
          bne       mag33               ;Else must be Plot
          bsr       getstr
          bne       mag3rep             ;error cond., exit
          lea       size(pc),A3
          move.l    D7,(A3)
          bra       mag3rep
mag33     cmpi.w    #5,(A2)             ;plot or creal
          bne       mag34
          lea       sel_mand(pc),A4
          cmpi.b    #TRUE,(A4)
          beq       mag44               ;must be plot
          bsr       getstr
          bne       mag3rep
          lea       creal(pc),A3
          move.l    D7,(A3)
          bra       mag3rep
mag34     cmpi.w    #6,(A2)
          bne       mag44               ;plot julia (7)
          bsr       getstr
          bne       mag3rep
          lea       cimag(pc),A3
          move.l    D7,(A3)
          bra       mag3rep
mag44     moveq     #5,D7               ;now draw mandelbrot
          rts

getstr    move.l    A0,-(A7)
          bsr       opencon
          bsr       inputcon
          move.l    A1,A5
          bsr       closecon
          bsr       reset
          bsr       asc2int
          cmpi.w    #0,D0
          bge       getstrok
          bsr       printerr
getstrok  move.l    (A7)+,A0
          cmpi.w    #0,D0
          rts
risnumb   move.l    A0,-(A7)
          add.l     #20,A5
          lea       rp(pc),A1
          bsr       clearnumb
          lea       ip(pc),A1
          bsr       clearnumb
          lea       size(pc),A1         ;size no.
          bsr       clearnumb
          lea       sel_mand(pc),A4
          cmpi.b    #TRUE,(A4)          ;mand selected?
          beq       ris_exit
          lea       creal(pc),A1        ;creal
          bsr       clearnumb
          lea       cimag(pc),A1        ;cimag
          bsr       clearnumb
ris_exit  move.l    (A7)+,A0
          rts
clearnumb move.l    (A1),D7
          move.l    A5,-(A7)
          lea       zeronumb(pc),A1
          lea       buffer(pc),A5
          bsr       copya1a5
          bsr       reset
          bsr       int2asc
          move.l    A5,A1
          move.l    (A7)+,A5
          bsr       copya1a5
          add.l     #14,A5
          rts
copya1a5  moveq     #8,D0
numbloop  move.b    0(A1,D0.W),0(A5,D0.W)
          dbf       D0,numbloop
          rts

multmenu  lea       multvar(pc),A5
          lea       messmul(pc),A4
          clr.l     D7
          cmpi.w    #1,(A5)
          bne       multon
multoff   move.w    #0,(A5)
          move.l    #'OFF ',10(A4)
          rts
multon    move.w    #1,(A5)             ;switch it on
          move.l    #'ON  ',10(A4)
          rts

filemenu  lea       menufile(pc),A5
          bsr       print
          lea       curtoplim(pc),A2
          move.w    #2,(A2)
          lea       curbotlim(pc),A2
          move.w    #8,(A2)
          lea       cursorpos(pc),A2
          move.w    #2,(A2)
          bsr       select
          move.w    (A2),D7
          cmpi.w    #7,D7
          beq       file_x
          cmpi.w    #2,D7
          beq       delemenu
          cmpi.w    #3,D7
          beq       diremenu
          cmpi.w    #4,D7
          beq       formmenu
          lea       inhibit(pc),A5
          cmpi.w    #0,(A5)
          bne       filemenu
          cmpi.w    #5,D7
          beq       loadmenu
          cmpi.w    #6,D7
          beq       prinmenu
          cmpi.w    #8,D7
          beq       savemenu
file_x    clr.l     D7
          rts

delemenu  lea       menudele(pc),A5
          move.w    #4,D0               ;IO.DELET
          move.l    A0,-(A7)
          bsr       io_oper
          move.l    (A7)+,A0
          bra       filemenu

formmenu  lea       menuform(pc),A5
          move.w    #3,D0               ;IO.FORMT
          move.l    A0,-(A7)
          bsr       io_oper
          move.l    (A7)+,A0
          tst.w     D0
          bne       filemenu            ;exit back if error
          movem.l   D1-D2/A0,-(A7)
          bsr       newpage
          movem.l   (A7)+,D1-D2/A0
          move.l    A0,A5
          bsr       printnum            ;no. of good sectors
          move.w    D2,D1
          bsr       printnum            ;out of total sectors
          moveq     #100,D1
          bsr       delay
          bra       filemenu

diremenu  lea       menudire(pc),A5
          moveq     #1,D0               ;IO.OPEN
          move.l    A0,-(A7)
          bsr       io_oper
          move.l    (A7)+,A5
          exg       A0,A5
          tst.w     D0
          bne       filemenu
          bsr       newpage
          exg       A0,A5
          lea       buffer(pc),A1
          moveq     #-1,D3
          moveq     #$45,D0
          trap      #3
          tst.w     D0
          bne       direexit
          swap      D1
          bsr       printnum
          swap      D1
          bsr       printnum
          lea       buffer(pc),A1
          moveq     #10,D1
          bsr       printstr
          bsr       printlf

file_name move.w    #64,D2
          lea       buffer(pc),A1
          moveq     #-1,D3
          moveq     #3,D0
          trap      #3
          tst.w     D0
          blt       dire0exit
          lea       buffer(pc),A1
          add.l     #$E,A1
          move.w    (A1)+,D1
          cmpi.w    #0,D1
          beq       file_name
          move.w    D1,D7
          divu      #14,D7
          andi.l    #7,D7
          addq.w    #1,D7
          bsr       testend
          add.w     D7,D6
          cmpi.w    #9,D6
          bls       disp_str
          bsr       pageend
          lea       buffer(pc),A1
          add.l     #$E,A1
          move.w    (A1)+,D1
disp_str  bsr       printstr
          bsr       printlf
          bra       file_name

dire0exit bsr       pageend
          clr.l     D0
direexit  tst.w     D0
          bge       ok_direx
          exg       A0,A5
          bsr       printerr
          exg       A0,A5
ok_direx  bsr       closefile
          exg       A0,A5
          bra       filemenu

testend   movem.l   D1/D7/A1,-(A7)
          exg       A0,A5
          move.w    #-1,D3
          lea       buffer(pc),A1
          moveq     #$B,D0
          trap      #3
          move.w    6(A1),D6
          exg       A0,A5
          movem.l   (A7)+,D1/D7/A1
          rts
pageend   exg       A0,A5
          moveq     #0,D1
          moveq     #9,D2
          bsr       positcur
          move.w    #14,D1
          lea       commbott(pc),A1
          exg       A0,A5
          bsr       printstr
          exg       A0,A5
          moveq     #8,D1
          bsr       delay
          bsr       key_enter
          bsr       newpage
          exg       A0,A5
          rts
newpage   moveq     #0,D1               ;menu chan ID / clear Menu
          moveq     #1,D2
          bsr       positcur
          moveq     #$20,D0             ;SD.CLEAR
          moveq     #-1,D3
          trap      #3
          clr.l     D6
          rts
positcur  moveq     #-1,D3
          moveq     #$10,D0             ;SD.POS position cursor
          trap      #3
          rts

key_enter movem.l   D1/D7,-(A7)         ;wait till enter key
key_rep   bsr       keyrow1
          cmpi.w    #1,D1
          bne       key_rep
          movem.l   (A7)+,D1/D7
          rts

io_oper   move.l    D0,-(A7)
          bsr       print
          bsr       opencon
          bsr       inputcon
          bsr       closecon
          lea       buffer(pc),A0
          moveq     #-1,D1
          moveq     #4,D3
          move.l    (A7)+,D0
          trap      #2
          tst.w     D0
          bge       ok_oper
          bsr       printerr
ok_oper   rts

printnum  exg       A0,A5               ;file ID to menus ID in A0
          movem.l   D1/D2,-(A7)
          andi.l    #255,D1
          move.w    $CE,A4              ;UT.MINT
          jsr       (A4)                ;print decimal number
          moveq     #SPACE,D1
          bsr       printchar
          movem.l   (A7)+,D1/D2
          exg       A0,A5               ;reset A0 to file chan.ID
          rts
printstr  movem.l   D1/D7,-(A7)
          exg       A0,A5
          bsr       prtstr
          exg       A0,A5
          movem.l   (A7)+,D1/D7
          rts
prtstr    move.w    D1,D2               ;number of bytes to send
          move.w    #-1,D3
          moveq     #7,D0               ;IO.SSTRG
          trap      #3                  ;send string at A1 to menu chan.
          rts
printlf   exg       A0,A5
          moveq     #LF,D1
          bsr       printchar
          exg       A0,A5
          rts

prinmenu  lea       menuprin(pc),A5     ;Screen dump to printer
          bsr       print
          move.l    A0,A5
          lea       curtoplim(pc),A4
          move.w    #7,(A4)
          lea       curbotlim(pc),A4
          move.w    #8,(A4)
          lea       cursorpos(pc),A2
          move.w    #7,(A2)
          bsr       select
          cmpi.w    #7,(A2)             ;confirm Print
          beq       printok
          cmpi.w    #8,(A2)             ;cancel this menu
          beq       print_ex

printok   move.w    #9600,D1
          moveq     #$12,D0
          trap      #1
          lea       serial1(pc),A0
          moveq     #2,D3
          bsr       openfile
          tst.w     D0
          bge       print1ok
          move.l    A5,A0
          bsr       printerr
          bra       filemenu            ;return
print1ok  bsr       swap
          moveq     #ESC,D1             ;ESC
          bsr       printchar
          moveq     #'A',D1
          bsr       printchar
          moveq     #8,D1
          bsr       printchar
          move.l    #131072,A2          ;base
          moveq     #0,D2               ;i=0 to 126 step 2
seriloop  move.l    A2,A3
          add.l     D2,A3               ;base+i
          add.l     #32768,A3           ;+128*pc
          moveq     #ESC,D1
          bsr       printchar
          moveq     #'*',D1
          bsr       printchar
          moveq     #5,D1
          bsr       printchar
          moveq     #0,D1
          bsr       printchar
          moveq     #2,D1
          bsr       printchar
          move.l    #255,D3             ;j=255 to 0
serjloop  sub.l     #128,A3             ;prints one line
          move.l    #255,D1
          sub.b     (A3),D1             ;addr
          bsr       printchar
          move.l    #255,D1
          adda.l    #1,A3
          sub.b     (A3),D1             ;addr+1
          suba.l    #1,A3
          bsr       printchar
          dbf       D3,serjloop
          moveq     #CR,D1
          bsr       printchar
          moveq     #LF,D1
          bsr       printchar
          addq.b    #2,D2               ;i+2
          cmpi.b    #128,D2
          bne       seriloop
          moveq     #2,D2
gaplines  moveq     #LF,D1
          bsr       printchar           ;shift printer 3 lines
          dbf       D2,gaplines
          bsr       closefile           ;IO.CLOSE ser1
          bsr       swap
print_ex  move.l    A5,A0
          bra       filemenu            ;return

printchar movem.l   D1-D4/A2/A3,-(A7)
          move.w    #-1,D3
          moveq     #5,D0               ;IO.SBYTE
          trap      #3                  ;send byte
          movem.l   (A7)+,D1-D4/A2/A3
          rts

loadmenu  move.l    A0,-(A7)
          lea       menuload(pc),A5
          bsr       print
          move.l    #0,D3
          bra       filename
savemenu  move.l    A0,-(A7)
          lea       menusave(pc),A5
          bsr       print
          move.l    #3,D3
filename  move.l    D3,-(A7)
          bsr       opencon
          bsr       inputcon
          bsr       closecon
          move.l    (A7),D3
          lea       buffer(pc),A0
          bsr       openfile
          tst.w     D0
          bge       errnot
          add.l     #4,A7
          bsr       printerr
          bra       restormen
errnot    bsr       swap
          move.l    (A7)+,D3
          tst.l     D3
          beq       loadflag
          move.l    #$20000,A5
          lea       rp(pc),A4
          move.l    (A4)+,(A5)+
          move.l    (A4)+,(A5)+
          move.l    (A4)+,(A5)+
          lea       scr_mand(pc),A2
          cmpi.b    #TRUE,(A2)
          beq       save1
          move.w    #$FFFF,(A5)+        ;flag=julia
          move.l    (A4)+,(A5)+
          move.l    (A4)+,(A5)+
save1     move.l    #$49,D0
          bsr       saveload
          tst.w     D0
          bge       saveok
          move.l    D0,-(A7)
          bsr       closefile
          bsr       swap
          move.l    (A7)+,D0
          bsr       printerr            ;error and exit?
          bra       restormen
saveok    bsr       closefile
          bra       clrflash
loadflag  move.l    #$48,D0
          bsr       saveload
          tst.w     D0
          bge       loadok
          move.l    D0,-(A7)
          bsr       closefile
          bsr       swap
          move.l    (A7)+,D0
          bsr       printerr
          bra       restormen
loadok    bsr       closefile
          move.l    #$20000,A5
          lea       rp(pc),A4
          move.l    (A5)+,(A4)+
          move.l    (A5)+,(A4)+
          move.l    (A5)+,(A4)+
          lea       scr_mand(pc),A2
          lea       sel_mand(pc),A3
          move.b    #TRUE,(A2)
          move.b    #TRUE,(A3)
          cmpi.w    #$FFFF,(A5)+
          bne       clrflash
          move.l    (A5)+,(A4)+
          move.l    (A5)+,(A4)+
          move.b    #FALSE,(A2)
          move.b    #FALSE,(A3)
clrflash  move.l    #$20000,A5
          moveq     #10,D4
clrfl1    move.w    $80(A5),(A5)+
          dbf       D4,clrfl1
          bsr       swap
restormen move.l    (A7)+,A0
          bra       filemenu

** micro filing procedures **
openfile  move.l    #-1,D1
          move.l    #1,D0
          trap      #2
          rts
saveload  cmpi.w    #$48,D0
          bne       headskip
          lea       header(pc),A1
          move.w    #64,D2
          move.w    #-1,D3
          move.l    #$47,D0
          trap      #3
          tst.w     D0
          blt       errhead
          lea       header(pc),A1
          cmpi.l    #$8000,(A1)
          beq       headread
          move.l    #-19,D0
          bra       errhead
headread  move.l    #$48,D0
headskip  move.l    #$8000,D2
          move.l    #$20000,A1
          move.w    #-1,D3
          trap      #3
errhead   rts
closefile move.l    #2,D0
          trap      #2
          rts

*** move highlight bar up and down Menus ***
* Entry - cursorpos = A2 : curtoplim=A3 - curbotlim=A1
select    move.l    A0,-(A7)
          lea       cursorpos(pc),A2
          move.w    (A2),D5
          bsr       highlight
untilmove moveq     #2,D1
          bsr       delay
          move.w    D5,-(A7)
          bsr       keyrow1
          move.w    (A7)+,D5
          cmpi.b    #1,D1
          beq       nextmenu
upnext    cmpi.b    #4,D1
          bne       downnext
          lea       curtoplim(pc),A3
          cmp.w     (A3),D5
          beq       untilmove
          bsr       highlight
          subq.w    #1,D5
          bsr       highlight
          bra       untilmove
downnext  cmpi.b    #128,D1
          bne       untilmove
          lea       curbotlim(pc),A3
          cmp.w     (A3),D5
          beq       untilmove
          bsr       highlight
          addq.w    #1,D5
          bsr       highlight
          bra       untilmove
nextmenu  move.l    (A7)+,A0
          lea       cursorpos(pc),A2
          move.w    D5,(A2)
          rts

highlight move.w    D5,-(A7)
          move.l    #$2C,D0
          move.w    #-1,D1
          move.w    #-1,D3
          trap      #3
          move.w    #10,D2
          move.w    (A7),D5
          mulu      D2,D5
          lea       block(pc),A1
          move.w    D5,6(A1)
          move.l    #$2E,D0
          move.b    #2,D1
          move.w    #-1,D3
          trap      #3
          move.w    (A7)+,D5
          rts

keyrow2   lea       cmmdrow(pc),A3
          move.b    #2,(A3)             ;row no. 2
          bra       getkey
keyrow1   lea       cmmdrow(pc),A3
          move.b    #1,(A3)             ;row no. 1
          bra       getkey
keyrow0   lea       cmmdrow(pc),A3
          move.b    #0,(A3)             ;row no. 0
getkey    lea       cmmd(pc),A3         ;cursor key row request
          moveq     #$11,D0             ;MT_IPCOM
          trap      #1
          rts                           ;value in D1

messF5    dc.b      "F5 SELECT "
          dc.b      191,190,'  '        ;down,up keys
          dc.b      'Accuracy      '
          dc.b      'File      MENU'
          dc.b      'Function      '
          dc.b      'Magnify   MENU'
          dc.b      'Move menu MENU'
messmul   dc.b      'Multitask ON  '
          dc.b      'Quit          '
          dc.b      'Restart       '
          dc.b      'Stop          $'
menuaccu  dc.b      'ACCURACY Menu @'
          dc.b      ' Use ',191,' ',190,' keys '
          dc.b      '  to change   '
          dc.b      ' value below  @@@@'
          dc.b      ' <key  Enter> $'
menufile  dc.b      'FILE Menu ',191,190,'  @'
          dc.b      'Delete    FILE'
          dc.b      'Directory DEV.'
          dc.b      'Format    DEV.'
          dc.b      'Load      FILE'
          dc.b      'Print     SCR.'
          dc.b      'Quit          '
          dc.b      'Save      FILE$'
menumagn  dc.b      'MAGNIFY Menu',191,190,'@'
          dc.b      'Location  INFO'
          dc.b      'Rectangle MENU'
          dc.b      'Values    MENU$'
menu1mag  dc.b      'LOCATION Info @'
          dc.b      'Real:         '
          dc.b      'Imag:         '
          dc.b      'Size:         @@@@'
          dc.b      ' <key  Enter> $'
menu2mag  dc.b      'RECTANGLE Menu@'
          dc.b      'Move   : ',191,190,188,189,' '
          dc.b      'Enlarge:   \  '
          dc.b      'Reduce : SPACE'
          dc.b      'Magnify:   M  @@@'
          dc.b      ' <key  Enter> $'
menu3mag  dc.b      'VALUES Menu ',191,190,'@'
          dc.b      'Real:         '
          dc.b      'Imag:         '
          dc.b      'Size:         '
          dc.b      'Plot          $'
menu4mag  dc.b      'MAGNIFY Menu',191,190,'@'
          dc.b      'Location  INFO'
          dc.b      'Pointer   MENU'
          dc.b      'Values    MENU$'
menu5mag  dc.b      'LOCATION Info @'
          dc.b      'Real:         '
          dc.b      'Imag:         '
          dc.b      'Size:         '
          dc.b      'CReal         '
          dc.b      'CImag         @@'
          dc.b      ' <key  Enter> $'
menu6mag  dc.b      'POINTER Menu  @'
          dc.b      'Move   : ',191,190,188,189,' '
          dc.b      'Magnify:   M  @@@@@'
          dc.b      ' <key  Enter> $'
menu7mag  dc.b      'VALUES Menu ',191,190,'@'
          dc.b      'Real:         '
          dc.b      'Imag:         '
          dc.b      'Size:         '
          dc.b      'CReal         '
          dc.b      'CImag         '
          dc.b      'Plot          $'
menumove  dc.b      'MOVE Menu     @'
          dc.b      'Move   : ',191,190,188,189,' '
          dc.b      'Place Menu: M @@@@@'
          dc.b      ' <key  Enter> $'
menudele  dc.b      'DELETE Menu   @'
          dc.b      'Type :        @'
          dc.b      'device_f.name $'
menudire  dc.b      'DIRECTORY Menu@'
          dc.b      'Type :        @'
          dc.b      'device_name   $'
commbott  dc.b      ' <key  Enter> '
menuform  dc.b      'FORMAT Menu   @'
          dc.b      'Type :        @'
          dc.b      'device_name   $'
menuload  dc.b      'LOAD Menu     @'
          dc.b      'Type :        @'
          dc.b      'device_f.name $'
menuprin  dc.b      'PRINT Menu ',191,190,' @'
          dc.b      'Screen dump to'
          dc.b      'Epson printer.'
          dc.b      'Set printer to'
          dc.b      'On-Line.      @'
          dc.b      'Print         '
          dc.b      'Cancel        $'
menusave  dc.b      'SAVE Menu     @'
          dc.b      'Type :        @'
          dc.b      'device_f.name $'
space14   dc.b      '              '    ;used by Print inplace of @ char
multvar   ds.w      1
inhibit   ds.w      1
cursorpos ds.w      1
curtoplim ds.w      1
curbotlim ds.w      1
block     dc.w      170
          dc.w      10
          dc.w      1
          ds.w      1
chanwin   dc.b      4,1
          dc.b      0,4
          dc.w      176,102
wincoord  dc.w      57,0
chancon   dc.b      2,1
          dc.b      0,5
          dc.w      173,24
concoord  dc.w      58,77
main_win  dc.b      0,0
          dc.b      0,0
          dc.w      511,255
          dc.w      0,0
mainchan  ds.l      1
buffer    ds.w      1
          ds.b      64
zeronumb  dc.b      ' 000000000'        ;used by clearnumb
mandsel   dc.b      'Mand'
julisel   dc.b      'Juli'
serial1   dc.w      4
          dc.b      'ser1'
cmmd      dc.b      9
          dc.b      1
          dc.l      0
cmmdrow   ds.b      1
          dc.b      2
sound     dc.b      $0A,8
          dc.l      $0000AAAA
          dc.b      12,0,0,0
          dc.b      0,4,0,0
          dc.b      1
          ds.b      1
header    ds.b      64        ;for file headers eg. checking if valid file
menubuff  ds.b      4488      ;over 4K buffer for window storage

printerr  movem.l   A0-A3,-(A7)
          move.l    D0,-(A7)
          lea       sound(pc),A3
          moveq     #17,D0
          trap      #1
          lea       chancon(pc),A1
          move.w    $C8,A0
          jsr       (A0)
          move.l    (A7),D0
          move.w    $CC,A2
          jsr       (A2)
          bsr       closecon
          move.l    #100,D1
          bsr       delay
          move.l    (A7)+,D0
          movem.l   (A7)+,A0-A3
          rts

opencon   lea       chancon(pc),A1      ;open console channel
          move.w    $C6,A0
          jsr       (A0)
          rts
inputcon  move.l    A3,-(A7)
          move.l    #28,D2              ;buffer size
          move.w    #-1,D3
          lea       buffer(pc),A1
          add.l     #2,A1
          move.l    #2,D0
          trap      #3
          lea       buffer(pc),A1
          sub.w     #1,D1
          move.w    D1,(A1)
          move.l    (A7)+,A3
          rts
closecon  move.l    #2,D0
          trap      #2
          rts

delay     move.l    #$FFF,D6
          move.l    D7,-(A7)
pause     move.l    D1,D7
ret       dbf       D7,ret
          dbf       D6,pause
          move.l    (A7)+,D7
          rts
memory    ds.l      1       ;32KByte data area for Mandelbrot start screen
end
